#ifndef CACHEDFILE_H
#define CACHEDFILE_H

//#define XP_UNIX

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "smilplugin.h"

#define STATE_INITIALIZED   1
#define STATE_DOWNLOADED    2
#define STATE_FROM_CACHE    4
#define STATE_OPENED_WRITE  8
#define STATE_OPENED_READ  16

typedef struct file {
	char *filename;
	char *mimetype;
	uint32 filesize;
	uint32 actualsize;
	int state;
	char *url;
	int filehandler;
} file;

class cachedfiles {
      public:
	cachedfiles(class nsPluginInstance *, const char *url);
	~cachedfiles(void);
	
	/* Functions that create new cache files.
	 */
	int newfile(const char *, const char *mimetype=NULL, 
			uint32 size=0);

	/* Functions to find existing cache file entries.
	 */
	const char *findfilename(const char *);
	file *findfile(const char *);

	/* Opening and closing cache files.
	 */
	int open_cache_file(file *, int);
	int close_cache_file(file *);

	/* Iterate through the current file list
	 */
	file *cachedfiles::get_first_file(void);
	file *cachedfiles::get_next_file(void);

	/* Reading and writing cache files.
	 */
	int writefile(int32, int32, void *, const char *);

	/* Functions concerning the list of actual cache file list.
	 */
	int print_cached_file_list(void);

	pthread_mutex_t	cache_mutex;  /* FIXME: must write the support. */
	pthread_cond_t new_file_cond; /* FIXME: must write the support. */
      private:
	int setup_cachelist(int);
	int find_in_cache(file *);  /* Find file in HD cache. */
	int file_available(file *); /* The file is readí now. */
	char *cachedfiles::canonicalize_url(const char *url, char *buffer, size_t size);
	nsPluginInstance *instance;
	file *files;
	int nfiles;
	int maxfiles;
	int current;
	char *base_url;
};

#endif

