#include "layout.h"

/* This function is used to create the first (main) layout_window entry. 
 * The first window is the window which is presented us by the browser,
 * so this is theonly layout_window if the smil document does not create
 * newwinows with thelayout tag.
 */
layout_window::layout_window(NPWindow *window)
{
	XGCValues val;
	Window x_child_window;
	Window x_childs[16];
	unsigned int x_nwindows;

	state = STATE_INITIALIZING;
	name = NULL;
	fw = NULL;
	bw = NULL;
	npwindow = window;
	/*
	 * Initialization of X window related stuff may be different
	 * if this is a first layout_window.
	 */
#ifdef DEBUG
	if (!window) {
		printf("%40s: But this window is NULL.\n",
		       __PRETTY_FUNCTION__);
	}
#endif
	/* Let's find the display */
	x_ws_info = (NPSetWindowCallbackStruct *) window->ws_info;
	x_display = x_ws_info->display;
	/* Let's find the X Window */
	x_window = (Window) window->window;
	/* Let's find the grandparent we need. */
	XQueryTree(x_display, x_window, &x_root_window,
		   &x_parent_window, (Window **) x_childs, &x_nwindows);
	XQueryTree(x_display, x_parent_window, &x_root_window,
		   &x_parent_window, (Window **) x_childs, &x_nwindows);
	XQueryTree(x_display, x_parent_window, &x_root_window,
		   &x_parent_window, (Window **) x_childs, &x_nwindows);
	XQueryTree(x_display, x_parent_window, &x_root_window,
		   &x_parent_window, (Window **) x_childs, &x_nwindows);
	
	/* We create two context for the sake of speed */
	val.foreground = BlackPixel(x_display, DefaultScreen(x_display));
	val.background = WhitePixel(x_display, DefaultScreen(x_display));
	x_gc = XCreateGC(x_display, (Drawable) x_window,
			 GCForeground, &val);
	val.foreground = WhitePixel(x_display, DefaultScreen(x_display));
	val.background = BlackPixel(x_display, DefaultScreen(x_display));
	x_inv_gc = XCreateGC(x_display, (Drawable) x_window,
			     GCForeground, &val);
	/* The font */
	x_font = XQueryFont(x_display, XGContextFromGC(x_gc));
	x_fontheight = x_font->max_bounds.ascent +
	    x_font->max_bounds.descent + 2;

	npwindow = window;
	state = STATE_INITIALIZED;
}

/*
 * This function is called, when the smil document creates windows
 * with thelayout tag in the head.
 */
layout_window::layout_window(layout_window *first, const char *thisname, 
	       int thisx, int thisy, int thisdx, int thisdy)
{
	XGCValues val;
	int screen;
	
#ifdef DEBUG
	if (!first | !name | !dx | !dy)
		printf("%40s: Trouble with arguments.\n",
				__PRETTY_FUNCTION__);
#endif
	
	state = STATE_INITIALIZING;
	x = thisx; y = thisy; dx = thisdx; dy = thisdy;
	name = strdup(thisname);
	fw = NULL;
	bw = first;
	npwindow = first->npwindow;
	x_parent_window = first->x_parent_window;
	x_root_window = first->x_root_window;
	x_display = first->x_display;
	x_ws_info = first->x_ws_info;

#ifdef DEBUG
	if (!npwindow) {
		printf("%40s: No NPWindow found.\n",
		       __PRETTY_FUNCTION__);
	}
#endif

	/*
	 * We create a new window for the X Window system as a child
	 * window of the browser. This is not too nice to do, but this
	 * move is our only chance to display multiple media paralel.
	 */
	screen = DefaultScreen(x_display);
	x_window = XCreateSimpleWindow(
				x_display,
				x_parent_window,
				x, y, dx, dy, 0,
				BlackPixel(x_display, screen),
				WhitePixel(x_display, screen));
	// FIXME: These are just tests, the final version should
	// map a window when it is needed.
	XRaiseWindow(x_display, x_window);
	XMapWindow(x_display, x_window);

	x_ws_info = (NPSetWindowCallbackStruct *) npwindow->ws_info;

	val.foreground = BlackPixel(x_display, DefaultScreen(x_display));
	val.background = WhitePixel(x_display, DefaultScreen(x_display));
	x_gc = XCreateGC(x_display, (Drawable) x_window,
			 GCForeground, &val);

	val.foreground = WhitePixel(x_display, DefaultScreen(x_display));
	val.background = BlackPixel(x_display, DefaultScreen(x_display));
	x_inv_gc = XCreateGC(x_display, (Drawable) x_window,
			     GCForeground, &val);

	x_font = XQueryFont(x_display, XGContextFromGC(x_gc));
	x_fontheight = x_font->max_bounds.ascent +
	    x_font->max_bounds.descent + 2;
	
	/*
	 * We are ready.
	 */
	state = STATE_INITIALIZED;
	while (first->fw)
		first = first->fw;
	
	first->fw = this;
}


/***************************************************************************
 * class: layout
 *
 ***************************************************************************/
layout::layout(NPWindow *window)
{
	first_layout_window = new layout_window(window);
}

layout::~layout(void)
{
	layout_window *this_layout = first_layout_window;
	layout_window *next_layout;
	
	while(this_layout){
		next_layout = this_layout->forward();
		delete this_layout;
		this_layout = next_layout;
	}
}


		
		
